function T = ABCD_cone_tube(varargin) % SRR: Edited 1/23/17

% Based on Mason 1928, Keefe 1984, Benade 1968, Mapes-Riordan 1993, Lewis &
% Neely 2015

% This program calculates the ABCD matrix for a conical segment of small 
% area variation. This can be used for any geometry with plane or spherical 
% wave propagation (e.g. concentric cone) by specifying the wall perimeter 
% (default: perimeter assumes area is that of a circle). When the area does
% not change, these calculations simplify to plane-wave propagation (e.g.
% cylindrical tube)

% NOTE: Many such segments must be concatenated to approximate a lossy cone
% with area variation, because Kappa is fixed for each segment

% Inputs: 
% f = frequency in Hz
% A_left = left-side area
% A_right = right-side area
% L = segment length
% P_smallEnd = perimeter (wall loss for non-standard geometry; set to 0 for
% lossless wave propagation)

% Output:
% ABCD matrix 'T'

if nargin == 4  
    f = varargin{1};
    d_left=2*sqrt(varargin{2}/pi);
    d_right =2*sqrt(varargin{3}/pi);
    L = varargin{4};
    P_smallEnd = pi*min(d_left,d_right);
elseif nargin == 5
    f = varargin{1};
    d_left=2*sqrt(varargin{2}/pi);
    d_right =2*sqrt(varargin{3}/pi);
    L = varargin{4};
    P_smallEnd = varargin{5};
else
    disp('Wrong number of inputs!'); return
end

PC = getPhysicalConstants;
k0 = 2*pi*f/PC.c;

if d_right>=d_left % make sure the model is oriented correctly
    d0 = d_left; d1 = d_right;
else d0 = d_right; d1 = d_left; 
end

A0 = pi*(d0/2)^2; 
r0 = PC.rho*PC.c/A0;    

correctionFactor = (pi*d0)/P_smallEnd; % correct for non-circular perimeter
% rv taken from Mapes-Riordan 1993, based on Keefe's Kappa. The perimeter
% correction is based on Mason's Kappa (these Kappa calculations can be
% shown to be the same to a ratio of 1.045/1.085; this difference may be
% due to the choice of physical constants)
rv = sqrt(PC.rho*2*pi*f*A0/(PC.mu*pi))*correctionFactor; 
k = k0.*(1.045./rv + 1j*(1+1.045./rv));
z0 = r0*(1+0.369./rv - 1j*0.369./rv);

% NOTE: The following can be cast in terms of area as well. I have already
% worked with and debugged this diameter version, so I'm leaving it as-is
A = sqrt(d1/d0)*(cosh(k*L)-((d1-d0)./(k*L*d1)).*sinh(k*L));
B = (d0/d1)*z0.*sinh(k*L);
C =(d1/d0)*(1./z0).*(((d1-d0)^2./(k*L*d0*d1)).*cosh(k*L)+(1-((d1-d0)^2./(k.^2*L^2*d0*d1))).*sinh(k*L));
D = (d0/d1)*(cosh(k*L)+((d1-d0)./(k*L*d0)).*sinh(k*L));

 
if d_right < d_left % swap ABCD direction for decreasing cone
    if length(f)==1 % calculate at a single frequency
        T = [D B; C A];
    else % calculate at multiple frequencies
        T(1,1,:) = D; T(1,2,:) = B; 
        T(2,1,:) = C; T(2,2,:) = A;
    end
else
    if length(f) ==1
        T = [A B; C D];
    else
        T(1,1,:) = A; T(1,2,:) = B; 
        T(2,1,:) = C; T(2,2,:) = D;
    end
end